#-------------------------------------------------------------------------------
# Name:        Least Cost Transect Analysis LCTA - MASTER
# Purpose:      Version 1.2
#
# Author:      Maarten J. van Strien
# E-mail:	maarten@vstrien.nl
# Created:     02.05.2012
# Copyright: 
# Licence:
#-------------------------------------------------------------------------------

# Input variables
wrkspc = 'C:\\workspace' #Folder in which the input and output files are stored
cllsz = 10 #The cellsize of the resistance surfaces created and the landscape analysed in the transects.
stdyr = 'large_scale_study_area.shp' #A shapefile with one polygon representing the study area
le_table = 'lands_el.dbf' #A dbf file with all the landscape elements that will be incorporated in the analysis. The dbf-file should contain the columns 'shape', 'column' and 'landel'. The 'shape' column lists the shapefiles with the landscape elements. The 'column' column mentions the name of the column containing the attributes for the landscape elements. The 'landel' column lists the landscape elements per shapefile. Make sure the landel names are not longer than 10 characters and have unique names.
samp_locs = 'rufus_OA_min5_LargeScaleP.shp' #A point shapefile with all the locations of the populations.
sampID = 'NAME' # The column name in 'samp_locs' containing the unique numbers for every population.
hsf = [0,3,6] #A vector of specialization factors with which the weight is given to the matrix in the resistance surfaces: 2^fctr. The analysis will be performed for each of the specialization factors.
buffs = [50,100,250] #A vector of buffer distance of the transects. The analysis will be performed for each of the buffer widths.
prs_table = 'pairs_5plus.dbf' #A dbf file with all the population pairs. The column indicating one population from the pair should be called "NODE1" and the column indicating the other population should be called "NODE2".
pythonPath = 'C:\\Python25\\python.exe' #The path of the python.exe
prep_slave = 'C:\\workspace\\LCTA_v1-0_PREP_SLAVE.py' #The path to the SLAVE script in which the resistance surfaces are created (i.e. preporatory steps).
lcp_slave = 'C:\\workspace\\LCTA_v1-0_LCP_SLAVE.py' #The path to the SLAVE script in which LCTA is performed.
str_l_slave = 'C:\\workspace\\LCTA_v1-0_STR_LINE_SLAVE.py' #The path to the SLAVE in which straight line transects are calculated.

# importing functions
from dbfpy import dbf
import time
import os
import arcgisscripting
import sys
import subprocess
import psutil

# Setting the initial environment
gp = arcgisscripting.create()
gp.Workspace = wrkspc
gp.Cellsize = cllsz
gp.SetProduct('ArcInfo')
gp.CheckOutExtension('Spatial')
gp.Overwriteoutput = 1

# Create and open a log.txt file
log_name = "log-"+time.strftime("%d-%m-%y-%H-%M", time.localtime())+".txt"

# Print to main window:
print "Starting LCTA run at "+time.strftime("%H:%M:%S", time.localtime())+" on "+time.strftime("%d %b %Y", time.localtime())
sys.stdout.flush()

# Print initialization settings to log file
log = open(wrkspc+"\\"+log_name, 'a')
log.write("!! Starting LCTA run at "+time.strftime("%H:%M:%S", time.localtime())+" on "+time.strftime("%d %b %Y", time.localtime())+"\n")
log.write("!! Cell size: "+cllsz.__str__()+"\n")
log.write("!! Study area shapefile: "+stdyr+"\n")
log.write("!! Deme locations: "+samp_locs+"\n")
log.write("!! Suitability factors: "+hsf.__str__()+"\n")
log.write("!! Buffer sizes: "+buffs.__str__()+"\n")
log.write("\n")
log.close()

# read the dbf file with all the landscape elements in it.
le_df = dbf.Dbf(wrkspc+"\\"+le_table)

# Create a table in which the results of LCTA will be stored in each loop
gp.CreateTable_management(wrkspc, "out_tab.dbf")
gp.AddField("out_tab.dbf","node1","TEXT")
gp.AddField("out_tab.dbf","node2","TEXT")
gp.AddField("out_tab.dbf","length","DOUBLE")
gp.AddField("out_tab.dbf","cost","DOUBLE")
gp.AddField("out_tab.dbf","all_pix","DOUBLE")
for rec in le_df:
    gp.AddField("out_tab.dbf", rec.__getitem__("LANDEL"), "DOUBLE")

# Saving the start physical memory used.
start_mem = int(psutil.phymem_usage().used)

# Make an le_list of which the first element is the default for sf = 0
le_list = []

# Set up the paramPrepSlave to run the Preparatory Slave script.
for rec in le_df:
    shpfl = rec.__getitem__("SHAPE")
    clmn = rec.__getitem__("COLUMN")
    le = rec.__getitem__("LANDEL")
    le_list.append(le)
    for sf in hsf:
        # Call the PREP_SLAVE to do the preporatory steps.
        paramPrepSlave = []
        paramPrepSlave.append(pythonPath)
        paramPrepSlave.append(prep_slave)
        paramPrepSlave.append(wrkspc) # sys.argv[1]
        paramPrepSlave.append(le_table) # sys.argv[2]
        paramPrepSlave.append(log_name) # sys.argv[3]
        paramPrepSlave.append(str(cllsz)) # sys.argv[4]
        paramPrepSlave.append(stdyr) # sys.argv[5]
        paramPrepSlave.append(str(sf)) # sys.argv[6]
        paramPrepSlave.append(shpfl) # sys.argv[7]
        paramPrepSlave.append(clmn) # sys.argv[8]
        paramPrepSlave.append(le) # sys.argv[9]

        subprocess.call(paramPrepSlave)

        del paramPrepSlave

print "Finished the preporatory steps"
sys.stdout.flush()

# Start of the main loop of LCTA
for buff in buffs:
    for fctr in hsf:
        # Call the Slave to calculate straight line transects with different buffer widths.
        if fctr == 0:
            paramSLSlave = []
            paramSLSlave.append(pythonPath) # first parameter is the name of the Python executable
            paramSLSlave.append(str_l_slave) # second parameter is the full path of the Python script
            paramSLSlave.append(wrkspc) # sys.argv[1]
            paramSLSlave.append(le_table) # sys.argv[2]
            paramSLSlave.append(log_name) # sys.argv[3]
            paramSLSlave.append(prs_table) # sys.argv[4]
            paramSLSlave.append(samp_locs) # sys.argv[5]
            paramSLSlave.append(sampID) # sys.argv[6]
            paramSLSlave.append(str(buff)) # sys.argv[7]
            paramSLSlave.append(str(cllsz)) # sys.argv[8]

            # Call the subprocess
            subprocess.call(paramSLSlave)

            del paramSLSlave

            # Make a list with parameters to be passed on to the SLAVE program for sf = 0
            paramLCPSlave = []
            paramLCPSlave.append(pythonPath) # first parameter is the name of the Python executable
            paramLCPSlave.append(lcp_slave) # second parameter is the full path of the Python script
            paramLCPSlave.append(wrkspc) # sys.argv[1]
            paramLCPSlave.append(le_table) # sys.argv[2]
            paramLCPSlave.append(log_name) # sys.argv[3]
            paramLCPSlave.append(prs_table) # sys.argv[4]
            paramLCPSlave.append(samp_locs) # sys.argv[5]
            paramLCPSlave.append(sampID) # sys.argv[6]
            paramLCPSlave.append(le_list[0]) # sys.argv[7]
            paramLCPSlave.append(str(buff)) # sys.argv[8]
            paramLCPSlave.append(str(fctr)) # sys.argv[9]
            paramLCPSlave.append(str(cllsz)) # sys.argv[10]

            # Call the subprocess
            subprocess.call(paramLCPSlave)

            del paramLCPSlave
        # If fctr is higher than 0, then it has to be performed for all the different landscape elements
        else:
            for rec in le_df:
                # Sets the migration habitat landscape for this run
                le = rec.__getitem__("LANDEL")

                # Make a list with parameters to be passed on to the SLAVE program
                paramLCPSlave = []
                paramLCPSlave.append(pythonPath) # first parameter is the name of the Python executable
                paramLCPSlave.append(lcp_slave) # second parameter is the full path of the Python script
                paramLCPSlave.append(wrkspc) # sys.argv[1]
                paramLCPSlave.append(le_table) # sys.argv[2]
                paramLCPSlave.append(log_name) # sys.argv[3]
                paramLCPSlave.append(prs_table) # sys.argv[4]
                paramLCPSlave.append(samp_locs) # sys.argv[5]
                paramLCPSlave.append(sampID) # sys.argv[6]
                paramLCPSlave.append(le) # sys.argv[7]
                paramLCPSlave.append(str(buff)) # sys.argv[8]
                paramLCPSlave.append(str(fctr)) # sys.argv[9]
                paramLCPSlave.append(str(cllsz)) # sys.argv[10]

                # Call the subprocess
                subprocess.call(paramLCPSlave)

                # Writing the amount of memory in use to the log file
                end_mem = int(psutil.phymem_usage().used)
                log = open(wrkspc+"\\"+log_name, 'a')
                log.write("\n")
                log.write("&& Memory in use in relation to start memory: "+("%.2f" % (end_mem - start_mem))+"\n")
                log.close()

                del paramLCPSlave

# Close open data
le_df.close()
gp.delete_management("out_tab.dbf")
del le_list

# Print finish message to log flie
log = open(wrkspc+"\\"+log_name, 'a')
log.write("\n")
log.write("!! Finished LCTA run at "+time.strftime("%H:%M:%S", time.localtime())+" on "+time.strftime("%d %b %Y", time.localtime())+"\n")
log.close()

# Print finish message to screen
print "Finished LCTA run at "+time.strftime("%H:%M:%S", time.localtime())+" on "+time.strftime("%d %b %Y", time.localtime())
